#!/bin/bash
set -u

[ $# -ne 0 ] || {
  cat <<USAGE
Usage:
worker.sh N [TOP_DIR]

N - algorithm number:
  1 - overwrite
  2 - store to temporary file and rename
  3 - store to temporary file, sync and then rename

TOP_DIR - optional, directory to write files into. Defaul value is "./workdirN".

Author: Volodymyr M. Lisivka <vlisikva@gmail.com>

USAGE
  exit 0
}

datagen() {
  local ITERATION="$1"

  local NUMBER_OF_PATTERNS=$(( (RANDOM%1000)+1 ))
  local PATTERN="$RANDOM$RANDOM$RANDOM"

  # Print header
  echo -n "Iteration: ${ITERATION}_"
  echo -n "Number of patterns: ${NUMBER_OF_PATTERNS}_"
  echo -n "Pattern: ${PATTERN}_"

  local I
  for((I=0; I<$NUMBER_OF_PATTERNS; I++))
  do
    echo -n "${PATTERN}_"
  done
}

performance() {
  local ITERATION="$1"

  [ $((ITERATION % 100)) -eq 0 ] || {
    echo "$ITERATION iterations, $SECONDS seconds, $(( (ITERATION*100/SECONDS)/100 )).$(( (ITERATION*100/SECONDS)%100 )) iterations per second."
  }
}

worker1() {
  local ITERATION=0
  while true
  do
    for FILE in {1..100}
    do
      datagen "$ITERATION" >"$TOP_DIR/$FILE"
    done
    let ITERATION++
    performance "$ITERATION"
  done
}

worker2() {
  local ITERATION=0
  while true
  do
    for FILE in {1..100}
    do
      datagen "$ITERATION" >"$TOP_DIR/$FILE.tmp"
      mv -f "$TOP_DIR/$FILE.tmp" "$TOP_DIR/$FILE"
    done
    let ITERATION++
    performance "$ITERATION"
  done
}

worker3() {
  local ITERATION=0
  while true
  do
    for FILE in {1..100}
    do
      datagen "$ITERATION" >"$TOP_DIR/$FILE.tmp"
      sync
      mv -f "$TOP_DIR/$FILE.tmp" "$TOP_DIR/$FILE"
    done
    let ITERATION++
    performance "$ITERATION"
  done
}

TOP_DIR="${2:-./workdir$1}"
mkdir -p "$TOP_DIR"

case "$1" in
  1)
    worker1
  ;;
  2)
    worker2
  ;;
  3)
    worker3
  ;;
esac
